/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.52a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_SH7086.c            (for RENESAS SH2 SH7086)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"
#include <iodefine.h>

/*********************************************************************
*
*                    Configuration
*
**********************************************************************
*/

#ifndef OS_FSYS
  /* internal peripheral clock frequency                            */
  /* depends on external oscillator and PLL settings used           */
  #define OS_FSYS (80000000L)   /* Standard setting is 80 MHz       */
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER (OS_FSYS / 16)
#endif
  
#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS)
#endif

/* Select UART for embOSView.
   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define it to be -1
   Currently the standard code below supports CPU internal UART0 and 1
   (But can be easily changed to other UARTs)
*/
#ifndef   OS_UART
  #define OS_UART 0
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 19200L
#endif

/******	End of configurable option **********************************/

#define OS_UART_USED (OS_UART == 0)

#if OS_UART_USED
  #if (OS_UART == 0)
    #define SCIx SCI0
  #endif
#endif

/*********************************************************************
*
*		Local defines
*
**********************************************************************
*/

#define CMSTR   (*(volatile OS_U16*) 0xFFFFCE00)
#define CMCSR_0 (*(volatile OS_U16*) 0xFFFFCE02)
#define CMCNT_0 (*(volatile OS_U16*) 0xFFFFCE04)
#define CMCOR_0 (*(volatile OS_U16*) 0xFFFFCE06)

#define STBCR4  (*(volatile OS_U8* ) 0xFFFFE808)

/*********************************************************************
*
*		Local functions
*
**********************************************************************
*/

#if (OS_UART_USED)
/*********************************************************************
*
*		_IsrUartRxHandler()
*/
static void _IsrUartRxHandler(void) {
  if (SCIx.SCSSR.BIT.RDRF == 1) {  // Rx data available?
    SCIx.SCSSR.BIT.RDRF = 0;       // Clear interrupt flag
    OS_OnRx(SCIx.SCRDR);           // Process data
  }
}

/*********************************************************************
*
*		_IsrUartTxHandler()
*/
static void _IsrUartTxHandler(void) {
  if (OS_OnTx()) {
    SCIx.SCSCR.BYTE  &= ~(0x80); //  No more bytes to send, disable Tx-Int
  }
}

/*********************************************************************
*
*		_IsrUartErrHandler()
*/
static void _IsrUartErrHandler(void) {
  SCIx.SCSCR.BYTE  &= ~(0x50);   // disable Rx and Rx-Int
  while (SCIx.SCSSR.BYTE & 0x78) {
    SCIx.SCSSR.BYTE &= ~(0x78);  // Clear error and Rx data flags 
  }   
  SCIx.SCSCR.BYTE  |= 0x50;      // Reenable Rx and Rx-Int
}

#endif

/*********************************************************************
*
*		_IsrTickHandler()
*/
static void _IsrTickHandler(void) {
  while ((CMCSR_0 & (1 << 7)) != 0) {
	CMCSR_0 &= ~(1 << 7);
  }
  OS_HandleTick();
}

/*********************************************************************
*
*		Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*		    OS_InitHW()
*
*  Initialize the hardware required for the OS to run. This will work
*  on any target hardware, but may have to be tailored a little
*  (regarding the clock frequency). Of course the same holds true
*  if for some reason you choose to use an other timer.
*  Per default we use the Compare match timer channel 0, which is an up-counter
*/

#define TIMER_RELOAD (OS_PCLK_TIMER / 1000)
void OS_InitHW(void) {
  OS_IncDI();                       // For Safety disable interrupts
  //
  // Enable module clock for CMT and initialize CMT
  //
  STBCR4 &= ~(1 << 5);              
  CMSTR  &= ~(1 << 0);              // Stop counter channel 0
  CMCOR_0 = TIMER_RELOAD;           // Initialize compare value
  CMCNT_0 = 0;                      // Reset counter
  CMCSR_0 = 0                       // Set divider to 1/8
          | (1 << 6)                // Compare match interrupt enable
		  ;
  INTC.IPRJ.BIT._CMT0 = 1;          // Set interrupt priority, enable lowest
  
  CMSTR  |= (1 << 0);               // Start timer
  OS_COM_Init();                    // Initialize UART for embOSView
  OS_SetFastIntPriorityLimit(8);    // Set Priority limit for "Fast" interrupts 
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {          // Idle loop: No task is ready to exec
  for (;;);                   // Nothing to do ... wait for a interrupt
}

/*********************************************************************
*
*       Get time [cycles]
*
*  This routine is required for task-info via embOS Viewer.
*  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;
  
  Time = OS_GetTime32();
  Cnt  = CMCNT_0;
  //  
  // Check if timer interrupt pending ...
  // 
  if ((CMCSR_0 & (1 << 7)) != 0) {  // Adjust result
    Cnt = CMCNT_0;
    Time++;
  } 
  return (TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*  Convert Cycles into micro seconds. When using 20.0 MHz, this
*  means dividing by 20. If you have a different clock frequency,
*  you may have to modify this routine in order to get proper
*  diagonstics. Please note: Operation of theOS is not affected
*  by this routine. It is used for profiling and testing
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick(), the embOS timer interrupt handler
*/
#pragma interrupt OS_ISR_Tick
void OS_ISR_Tick(void) {
  OS_CallISR(_IsrTickHandler);
}

/*********************************************************************
*
*       Communication for embOSView   (UART 0)
*
**********************************************************************
*/

#if OS_UART_USED
#if (OS_UART == 0)
  #define BAUDPRESCALE 0  // CKE bits: 0,1,2,3
  #define BAUDDIVIDE ((OS_FSYS+(OS_BAUDRATE*16L))/(OS_BAUDRATE*32L))-1
#endif
  
/*********************************************************************
*
*       OS_COM_Init()
*/
void OS_COM_Init(void) {
  int i;

  OS_IncDI();                           // Disable interrupts, save previous state
  STB.CR3.BIT._SCI0 = 0;                // Enable SCI module
  //
  // Initialize port pin function
  //
  PFC.PAIORL.BIT.B1 = 1;
  PFC.PACRL1.BIT.PA1MD = 1;
  PFC.PACRL1.BIT.PA0MD = 1;
  //
  // Initially clear UART configuration
  //
  SCI0.SCSCR.BYTE = 0;
  SCI0.SCSSR.BYTE = 0;
  //
  // Setup communication format
  // 8bit data, None parity bit, 1 stop bit, P/1 clock */
  SCI0.SCSMR.BYTE = 0;
  SCI0.SCBRR = 0x40;        // Baudrate, right now fixed to 19200
  for(i=0 ; i<3000 ; i++) {
  }
  //
  // Enable interrupt, lowest priority
  //
  INTC.IPRL.BIT._SCI0 = 0x01;
  SCI0.SCSCR.BYTE = 0x70;               // Enable Rx interrupt, enable Rx Tx
  OS_DecRI();                           // Restore interrupt state
}

/*********************************************************************
*
*       OS_COM_Send1()
*/
void OS_COM_Send1(OS_U8 c) {
  SCIx.SCTDR = c;
  SCIx.SCSSR.BIT.TDRE = 0;       // Clear Tx Empty Flag
  SCIx.SCSCR.BYTE  |= 0x80;      // Enable Tx-Int
}

/*********************************************************************
*
*       OS_ISR_rx()
*/
#pragma interrupt OS_ISR_rx
void OS_ISR_rx(void) {
  OS_CallNestableISR(_IsrUartRxHandler);
}

/*********************************************************************
*
*       OS_ISR_tx()
*/
#pragma interrupt OS_ISR_tx
void OS_ISR_tx (void) {
  OS_CallNestableISR(_IsrUartTxHandler);
}

/*********************************************************************
*
*       OS_ISR_RxErr()
*/
#pragma interrupt OS_ISR_RxErr
void OS_ISR_RxErr(void) {
  OS_CallNestableISR(_IsrUartErrHandler);
}

#else  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy  */
  }
#endif /* (OS_UART_USED)

/****** End of file *************************************************/
