/***********************************************************************************
FILE NAME  	: HMONConfigUser.c

DESCRIPTION	: Configuration options for the HMON Monitor.
			  This file contains configuration options which you should customise for your
			  system device type, host to target communications, and overall behaviour.

CPU Type	: SH7086

Copyright   : 2006 Renesas Technology Europe Ltd.
			  2006 Renesas Technology Corporation.
			  All Rights Reserved
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
06.12.2006 RTE-NPS First Release.
***********************************************************************************/

/**********************************************************************************
System Includes
***********************************************************************************/

#include <machine.h>

/**********************************************************************************
User Includes
***********************************************************************************/
#include "HMONConfigUserStruct.h"
#include "HMONConfigUser.h"

/***********************************************************************************
Compiler Directives
***********************************************************************************/
/* This #pragma ensures the structure is kept seperate to the user's area
   and allows monitor to be protected whilst user code changes through the
   GUI memory mapping (user controlled). */
#pragma section HMON

/**********************************************************************************
Function Macros
***********************************************************************************/

                                    /* Define a function macro to prevent the
                                       unused variable warning from the compiler
                                       when we know that the variable is un-used. */
#define AVOID_UNUSED_WARNING(p)     if ((unsigned long)p & 0)

/**********************************************************************************
Constant Data
***********************************************************************************/
const _HMONCapabilityStructureUser _HMONCapabilityUser = {

	/* Pointer to a pointer to the start address of the
       user program for a power-on reset */
	HMON_POWER_ON_RESET_PTR_PTR,

	/* Address that the stack pointer should be set to
       following a power-on reset */
	HMON_POWER_ON_RESET_STACK_PTR,

	/* Pointer to a pointer to the start address of the
       user program for a manual reset */
	HMON_MANUAL_RESET_PTR_PTR,

	/* Address that the stack pointer should be set to following a
       manual reset */
	HMON_MANUAL_RESET_STACK_PTR,

	/* This should be set to the software breakpoint instruction (TRAPA or
       similar). The monitor uses this to perform PC breakpoints in RAM code,
       by replacing the instruction at the location with this opcode. */
	HMON_SOFTWARE_BREAKPOINT_OPCODE,

	/* This should be set to the opcode of the compiled-in
       software break instruction
     */
	HMON_COMPILED_IN_BREAKPOINT_OPCODE,

	/* Default HMON interrupt level, default this is set to one less than the
       maximum for the chip to enable the monitor to debug interrupt routines.
     */
	HMON_DEFAULT_INTERRUPT_LEVEL,

	/* This setting determines how the monitor starts up (whether monitor
       remains resident or user code is run straight away).
     */
    HMON_STARTUP_OPTION
};

/**********************************************************************************
Public Functions
***********************************************************************************/
/**********************************************************************************
Function Name: _HMONExtendedCommandHandler
Description:   Function for handling extended monitor commands. If the monitor
               recieves any commands from the host that it does not understand,
               it passes the data to this function.
Parameters:    IN  pucInputData - Pointer to the data received from the host
               IN  nSizeofInputData - The number of console input characters
               OUT ppucReturnData - Pointer to a pointer that will be set to the
                                    return data.
               OUT pnSizeofOutputData - Pointer to the number of characters
                                        contained in the reply
Return value:  none
**********************************************************************************/
void _HMONExtendedCommandHandler(UINT8 *pucInputData,
                                 INT16 nSizeofInputData,
                                 UINT8 **ppucReturnData,
                                 INT16 *pnSizeofReturnData)
{
                                    /* Delete the AVOID_UNUSED_WARNING function
                                       macro when the variables are used */
    AVOID_UNUSED_WARNING(pucInputData);
    AVOID_UNUSED_WARNING(nSizeofInputData);
                                    /* TODO: Handle any extended commands here */
    if (*pucInputData == 'z')
    {
        *ppucReturnData = (UINT8*)"OK";
        *pnSizeofReturnData = 2;
    }
    else
    {
        *pnSizeofReturnData = 0;
    }
}
/**********************************************************************************
End of function _HMONExtendedCommandHandler
***********************************************************************************/

/**********************************************************************************
Function Name: _HMONConsoleInputHandler
Description:   Stub function for handling console input. When console input is
               received from the host, HMON will call this function to handle it.
Parameters:    I/O pucBuffer - Pointer to the character array
               IN  nSizeofInputData - The number of console input characters
               OUT pnSizeofOutputData - The number of characters contained in the
                                        reply
Return value:  none
**********************************************************************************/
void _HMONConsoleInputHandler(UINT8 *pucBuffer,
                              INT16 nSizeofInputData,
                              INT16 *pnSizeofOutputData)
{
                                    /* Delete the AVOID_UNUSED_WARNING function
                                       macro when the variables are used */
    AVOID_UNUSED_WARNING(nSizeofInputData);
#if 0
    UINT8 *pSrc;
    UINT8 *pDst;
    UINT8 value;
                                    /* Skip over 'o' command letter */
    pSrc = pucBuffer + 1;
    pDst = WriteDataArea;
    while(nSizeofInputData > 2)
    {
        _HMONConvertSingleCharToHexValue(*pSrc++, &value);
        *pDst = (value << 4);
        _HMONConvertSingleCharToHexValue(*pSrc++, &value);
        *pDst = *pDst + value;
        nSizeofInputData-=2;
        pDst++;
    }
#endif
    pucBuffer[0] = 'O';
    pucBuffer[1] = 'K';
    *pnSizeofOutputData = 2;
}
/**********************************************************************************
End of function _HMONConsoleInputHandler
***********************************************************************************/

/**********************************************************************************
Function Name: _HMONFatalError
Description:   This function is called when the monitor encounters an unrecoverable
               error (e.g. communications library failure)
Parameters:    IN  userpc - The PC at which it all went tits up
Return value:  none
**********************************************************************************/
void _HMONFatalError(UINT32 userpc)
{
   /* Delete the AVOID_UNUSED_WARNING function
 	macro when the variables are used */
    AVOID_UNUSED_WARNING(userpc);
    for(;;)
	{
    }
}
/**********************************************************************************
End of function _HMONFatalError
***********************************************************************************/

/**********************************************************************************
Function Name: _HMONConsoleOutput
Description:  Can be called by the user program when it wants to send a console
              output message to the host

Parameters:
    pucOutputData     - (IN) Pointer to the message to send
    nSizeofData       - (IN) The number of characters in the message
**********************************************************************************/

int _HMONConsoleOutput(UINT8 *pucOutputData, INT16 nSizeofData)
{
    trapa(HMON_CONSOLE_OUTPUT_TRAP);
}
/**********************************************************************************
End of function _HMONConsoleOutput
***********************************************************************************/

/**********************************************************************************
End  Of File
***********************************************************************************/
